use esp_idf_svc::hal::{
    self,
    peripheral::Peripheral,
    gpio::{AnyInputPin, AnyOutputPin, InputPin},
    units::Hertz,
    delay::BLOCK,
    uart::{self, Uart},
};

use crate::error::Error;

pub mod gpsdata;

#[derive(Debug)]
#[allow(unused)]
pub enum GpsError {
    ParseError(nmea_parser::ParseError),
    UpdateError(),
    ReadError(),
    NoData(),
}

pub struct GPS<'a> {
    uart: uart::UartRxDriver<'a>,
    latest: gpsdata::GpsData,
    current: gpsdata::GpsData
}

impl<'a> GPS<'a> {
    /// Returns a new GPS struct with interfaces to allow for fetching
    /// uart: Uart connection to GPS module.
    pub fn new(
        uart: impl Peripheral<P = impl Uart> + 'a,
        gps_rx: impl Peripheral<P = impl InputPin> + 'a,
    ) -> Result<GPS<'a>, Error> {
         // Setup UART to read serial data from GPS
        let config = uart::config::Config::default().baudrate(Hertz(9_600));
        let uart: uart::UartRxDriver = uart::UartRxDriver::new(
            uart,
            gps_rx,
            None::<AnyInputPin>,
            None::<AnyOutputPin>,
            &config
        ).map_err(|err| Error::EspError(err))?;

        // Create GPS struct
        Ok(GPS { 
            uart, 
            latest: gpsdata::GpsData::default(), 
            current: gpsdata::GpsData::default() 
        })
    }

    pub fn poll(self: &mut GPS<'a>) -> Result<&gpsdata::GpsData, Error> {
        //Read buffer from UART
        let mut buf: Vec<u8> = (
            0..self.uart.count().map_err(|err| Error::EspError(err))? as u8
        ).collect();
        self.uart.read(&mut buf[..], BLOCK).map_err(|err| Error::EspError(err))?;

        //Update GPS Data Struct with the latest data fetched from UART buffer
        self.latest.update(&buf)
        .map_err(|err| Error::GpsError(err))?;

        // Return the latest stored data
        Ok(&self.latest)
    }
}