#![allow(unused)]

use esp_idf_svc::hal::{
    self,
    gpio::*,
    i2c::{I2cConfig, I2cDriver},
    prelude::Peripherals,
    units::Hertz,
};

use std::sync::atomic::{AtomicBool, Ordering};

mod error;
use error::Error;

mod appstate;

mod gps;
use gps::{gpsdata::GpsData, GPS};

mod display;
use display::Display;
use ssd1306::prelude::*;

fn main() -> Result<(), Error> {
    // It is necessary to call this function once. Otherwise some patches to the runtime
    // implemented by esp-idf-sys might not link properly. See https://github.com/esp-rs/esp-idf-template/issues/71
    esp_idf_svc::sys::link_patches();
    // Bind the log crate to the ESP Logging facilities
    //esp_idf_svc::log::EspLogger::initialize_default();

    // Get Peripherals and sysloop ready
    let peripherals = Peripherals::take().unwrap();
    let pins = peripherals.pins;

    // Define program's pins + addresses
    let scl = pins.gpio22;
    let sda = pins.gpio21;
    let gps_rx = pins.gpio16;
    let _lcd_address: u8 = 0x3C;

    // Setup I2C driver
    let config = I2cConfig::new().baudrate(Hertz(921600));
    let i2c =
        I2cDriver::new(peripherals.i2c0, sda, scl, &config).map_err(|err| Error::EspError(err))?;

    // Setup GPIO interrupts
    static FLAG: AtomicBool = AtomicBool::new(false);
    let mut interrupt_button_handle = PinDriver::input(pins.gpio9).unwrap();

    interrupt_button_handle.set_pull(Pull::Up);
    interrupt_button_handle.set_interrupt_type(InterruptType::NegEdge);
    unsafe {
        interrupt_button_handle.subscribe(app_state_interrupt);
    }
    interrupt_button_handle.enable_interrupt();

    // Prepare components for processing
    let mut gps: GPS = GPS::new(peripherals.uart0, gps_rx)?;
    let mut display: Display<DisplaySize128x64> =
        Display::new(i2c, DisplaySize128x64, DisplayRotation::Rotate0)?;
    let mut _app_state = appstate::AppState::default();
    const NO_DATA: &str = "NO\nGPS\nDATA";

    loop {
        let text: String;
        // Get the latest data from GPS module and flush to Display
        text = match gps.poll() {
            Ok(res) => match res.get_speed() {
                Some(s) => format!("{:.2} MPH", s),
                None => NO_DATA.to_string(),
            },
            Err(e) => {
                eprintln!("{:?}", e);
                NO_DATA.to_string()
            }
        };
        display.draw(text.as_str())?
    }
}

fn app_state_interrupt() {
    println!("inteerrupt");
}
